#define VERSION "Mesure Vitesse sur Pro-Micro V1.0 du 17/06/16 - LOCODUINO - Dominique"

#include <TM1637Display.h>
#include <EEPROM.h>

// Tube module connection pins (Digital Pins)
#define CLK 2
#define DIO 3

TM1637Display display(CLK, DIO);

// detector pins
#define DT0 8
#define DT1 9

uint8_t data[] = { 0xff, 0xff, 0xff, 0xff };
uint8_t blank[] = { 0, 0, 0, 0 };
uint8_t lumin = 0x0f;
unsigned long _time1, _time2, temps; 
byte dt0, dt1, odt0, odt1;
bool debug = false;
bool done;

// etat des capteurs de proximité
// repos = tous les bits à 0 (etat=0)
// dt0 = 0 si détection sur DT0 => P0=1
// dt1 = 0 si détection sur DT1 => P1=2
// 1ere detection sur DT0 : DP0 = 4
// 1ere detection sur DT1 : DP1 = 8
// affiche pendant _time2 : AFF = 16
enum  {Repos=0, P0=1, P1=2, DP0=4, DP1=8, AFF=16 };
byte etat=Repos;

// distance entre capteurs 500 mm
unsigned int distance=500;
unsigned int VCM, VKM;

unsigned int val = 500;// distance par défaut en mm
unsigned int delai = 5000;
unsigned int k=0;
byte V0,V1;

// ----------------------------
//      SETUP
// ----------------------------

void setup() {

  Serial.begin(115200);
    while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }

  Serial.println(VERSION);
  //Serial.println("init tubes");
  display.setBrightness(0x0f); 
  // All segments on
  display.setSegments(data);
  delay(1000);
  Serial.println("init détecteurs");
  // init Pins détecteurs
  pinMode(DT0, INPUT_PULLUP);
  pinMode(DT1, INPUT_PULLUP);
  V0 = EEPROM.read(0);
  V1 = EEPROM.read(1);
  if ((V0!=0xFF)&&(V1!=0xFF)) {
    val = V0;
    val = val << 8;
    val = val | V1;
  }
  display.showNumberDec(val, false, 4, 0);
  distance=val;
  Serial.print("distance = ");Serial.println(distance);
  delay(2000);

  odt0=digitalRead(DT0);
  odt1=digitalRead(DT1);
  if (!odt0 && !odt1) { // les 2 detecteurs sont excités
    // reglage distance
    reglage();
  }
  _time1=millis();
  _time2=millis();
  etat=AFF;
}

// ---------------------------
//  Reglage distance
// ---------------------------

void reglage(void) {
done = false;
  Serial.println("reglage distance");
  Serial.print("distance initiale = ");Serial.println(val);
  delay(2000);
  odt0=digitalRead(DT0);
  odt1=digitalRead(DT1);
  while ((!odt0 || !odt1)) {
    // attente détecteurs aux repos tous les 2 
    odt0=digitalRead(DT0);
    odt1=digitalRead(DT1);
  }
  Serial.println("debut reglage");
  while (!done) {
    odt0=digitalRead(DT0);
    odt1=digitalRead(DT1);
    if (!odt0) { // augmente
      val++;
      display.showNumberDec(val, false, 4, 0);
      delay(200);
    } 
    if (!odt1) { // diminue
      val--;
      display.showNumberDec(val, false, 4, 0);
      delay(200);
    } 
    if (!odt0 && !odt1) { // enregistrement
      V1 = val;
      V0 = val >> 8;
      EEPROM.update(0, V0);
      EEPROM.update(1, V1);
      done=true;
      Serial.print("fin reglage distance = ");Serial.println(val);
      distance = val;
    }
  }
}
// ---------------------------
//  Calcul de vitesse
// ---------------------------

void calcul(void) {
  if (temps>0) {
  VCM=distance*100/temps;
  VKM=VCM*576/100;
  display.showNumberDec(VKM, false, 4, 0);
  Serial.print(", vitesse=");Serial.println(VKM);
  delay(delai);
  etat=AFF;
  _time2=millis();
  }
}


// ----------------------------
//      LOOP
// ----------------------------

void loop() {

  // lecture des capteurs de proximité
  
  dt0=digitalRead(DT0);
  if (!dt0) {             // detection sur DT0 (niveau LOW)
    if (etat==Repos) {          // =0 : premiere detection sur DT0
      etat = P0+DP0;      // 5
      k=0;
      temps = millis();
      _time1=millis();  
      if (debug) {Serial.print("etat=");Serial.println(etat);}
    } else {              // sinon c'est une 2me detection
      if (etat == P1+DP1) { // bonne seulement après DT1
        temps = millis()-temps;
        if (debug) {Serial.print(" Temps=");Serial.print(temps); }
        if (debug) {Serial.print(", Distance=");Serial.print(distance);}
        calcul();
      } 
    }
  }
  
  dt1=digitalRead(DT1);
  if (!dt1) {             // detection sur DT1 (niveau LOW)
    if (etat==Repos) {          // =0 : premiere detection sur DT0
      etat = P1+DP1;      // 10
      k=0;
      temps = millis();
      _time1=millis();  
      if (debug) {Serial.print("etat=");Serial.println(etat);}
    } else {              // sinon c'est une 2me detection 
      if (etat == P0+DP0) { // bonne seulement après DT0
        temps = millis()-temps;
        if (debug) {Serial.print(" Temps=");Serial.print(temps); }
        if (debug) {Serial.print(", Distance=");Serial.print(distance);}
        calcul();
      }
    }
  }

  //enum etat {Repos=0, P0=1, P1=2, DP0=4, DP1=8, AFF=16 };

  while (Serial.available() > 0) 
  {
    val = Serial.parseInt();  // lecture entier
    if (Serial.read() == '\n') 
    {
      if (val==0) {debug = !debug;}
      else {
        V1 = val;
        V0 = val >> 8;
        EEPROM.update(0, V0);
        EEPROM.update(1, V1);
        Serial.print("distance entre capteurs : ");
        Serial.println(val);
        distance = val;
      }
    }
  }

  if (_time2+delai > millis()) {
    if (etat==AFF) {
      etat=Repos;
      if (debug) {Serial.print("etat=");Serial.println(etat);}
      display.setSegments(blank);
    }
  }

  if ((etat==P1+DP1)||(etat==P0+DP0)) {
    display.showNumberDec(k, false, 4, 0);
    k++;
  }
} // fin de LOOP


